<?php

/**
 * @package plugin Bundler
 * @copyright (C) 2010-2012 RicheyWeb - www.richeyweb.com
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL
 *
 */
// no direct access
defined('_JEXEC') or die('Restricted access');

jimport('joomla.plugin.plugin');
jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');

/**
 * Bundler system plugin
 */
class plgSystemBundler extends JPlugin {

    private $_done = false;
    private $jsfiles = array();
    private $cssfiles = array();
    private $jsfile = false;
    private $cssfile = false;
    private $_version = '';
    private $_folder = '';

    function onBeforeCompileHead() {
        $this->_doc = JFactory::getDocument();
        $this->_app = JFactory::getApplication();
        if ($this->_app->isAdmin() || $this->_doc->getType() != 'html' || $this->_done) {
            return true;
        }
        $this->_setup();
        if ($this->_done) {
            return true;
        }
        $this->_process();
    }

    function _setup() {
        $this->jsfiles = $this->_getFiles($this->params->get('js', array(), 'ARRAY'));
        $this->cssfiles = $this->_getFiles($this->params->get('css', array(), 'ARRAY'));
        if (!count($this->jsfiles) && !count($this->cssfiles) && !(int)$this->params->get('force', 0)) {
            $this->_done = true;
            return true;
        }
        $this->_folder = $this->params->get('folder', '', 'STRING');
        $this->_version = $this->_doc->getMediaVersion() . '-' . $this->_configsum($this->_folder, $this->jsfiles, $this->cssfiles);
        $this->jsfile = JFile::exists(JPATH_ROOT . '/media/plg_system_bundler/js/' . $this->_version . '.js');
        $this->cssfile = JFile::exists(JPATH_ROOT . '/media/plg_system_bundler/css/' . $this->_version . '.css');
    }

    function _process() {
        if (count($this->jsfiles) && (!$this->jsfile || (int)$this->params->get('force', 0))) {
            $this->_combineFiles($this->jsfiles, 'js');
        }
        if (count($this->cssfiles) && (!$this->cssfile || (int)$this->params->get('force', 0))) {
            $this->_combineFiles($this->cssfiles, 'css');
        }

        if (count($this->jsfiles)) {
            $this->_processFiles($this->jsfiles,'js');
        }
        if (count($this->cssfiles)) {
            $this->_processFiles($this->cssfiles,'css');
        }
        $this->_done = true;
    }
    
    function _processFiles($files,$type) {
        $source = array('js'=>array('_scripts','addScript'),'css'=>array('_styleSheets','addStyleSheet'));
        $this->_removeItems(array_intersect(array_keys($files), array_keys($this->_doc->{$source[$type][0]})), $type);
        foreach ($this->_outputFiles($files) as $outfile) {
            if ($outfile !== 0) {
                $path = JURI::root(true) . '/media/plg_system_bundler/'.$type.'/' . $outfile . '.'.$type;
                $this->_doc->{$source[$type][1]}($path,array('version'=>'auto'));
                $this->_doc->{$source[$type][0]} = array($path => $this->_doc->{$source[$type][0]}[$path]) + $this->_doc->{$source[$type][0]};
            } else {
                $path = JURI::root(true) . '/media/plg_system_bundler/'.$type.'/' . $this->_version . '.'.$type;
                $this->_doc->{$source[$type][1]}($path);
                $this->_doc->{$source[$type][0]} = array($path => $this->_doc->{$source[$type][0]}[$path]) + $this->_doc->{$source[$type][0]};
            }
        }        
    }

    function _configsum($folder, $js, $css) {
        return md5($folder . json_encode($js) . json_encode($css));
    }

    function _outputFiles($a) {
        return array_unique(array_values($a));
    }

    function _getFiles($a) {
        $r = array();
        foreach ($a as $i) {
            $r[$i->file] = strlen(trim($i->output)) ? trim($i->output) : 0;
        }
        return $r;
    }

    function _combineFiles($files, $type) {
        foreach (JFolder::files(JPATH_ROOT . '/media/plg_system_bundler/' . $type . '/', $type) as $old) {
            JFile::delete(JPATH_ROOT . '/media/plg_system_bundler/' . $type . '/' . $old);
        }
        $fp = array();
        foreach ($files as $source => $dest) {
            $filename = $dest ? $dest . '.' . $type : $this->_version . '.' . $type;
            $fp[$filename] = fopen(JPATH_ROOT . '/media/plg_system_bundler/' . $type . '/' . $filename, 'a+');
            if (strlen(trim($this->_folder))) {
                $source = preg_replace('/^' . preg_quote(trim($this->_folder), '/') . '/', '', $source);
            }
            $content = file_get_contents(JPATH_ROOT . $source);
            if ($content) {
                fwrite($fp[$filename], $this->_compress($content, $type)."\n");
            } else {
                error_log('unable to read ' . $filename);
            }
        }
        foreach (array_keys($fp) as $key) {
            fclose($fp[$key]);
        }
    }

    function _compress($content, $type) {
        return $type=='css'?$this->_compressCSS($content):$this->_compressJS($content);
    }

    function _compressCSS($css) {
        // via https://davidwalsh.name/css-compression-php
        return str_replace('; ', ';', str_replace(' }', '}', str_replace('{ ', '{', str_replace(array("\r\n", "\r", "\n", "\t", '  ', '    ', '    '), "", preg_replace('!/\*[^*]*\*+([^/][^*]*\*+)*/!', '', $css)))));
    }
    
    

    function _compressJS($javascript) {
        // someday I'll put something reliable here, so far my attempts fail on poorly written js input - until then, returning raw source
        return $javascript;
    }

    private function _removeItems($items, $type) {
        $property = $type == 'js' ? '_scripts' : '_styleSheets';
        foreach ($items as $item) {
            unset($this->_doc->{$property}[$item]);
        }
    }

}
